<div>

<h2><a href="#book_{{ entity.entityId }}" name="book_{{ entity.entityId }}">{{ entity.title }}</a></h2>
<h3>{{ i18n_TABLE_OF_CONTENTS }}</h3>

<div class="toc">
{% include "book_toc_list.html" %}
</div>

<div>
{% for entity in entity.entities %}
  <hr />
  {% if entity.isBook %}
    {% include "booktemplate.html" %}
  {% else %}
    {% if entity.isPage %}
      {% include "pagetemplate.html" %}
    {% endif %}
  {% endif %}
{% endfor %}
</div>
</div>
