<!DOCTYPE kpartgui>
<kpartgui version="11" name="kjots" >
 <MenuBar>

  <Menu name="file">
   <Action name="new_page"/>
   <Action name="new_book"/>
   <Separator/>
   <Action name="save_to"/>
   <Action name="import"/>
   <Action name="del_page"/>
   <Action name="del_folder"/>
   <Separator/>
   <Action name="rename_entry"/>
   <Separator/>
   <Action name="file_print"/>
   <Action name="file_print_preview"/>
   <Separator/>
  </Menu>

  <Menu noMerge="1" name="format"><text>F&amp;ormat</text>
      <Action name="format_text_bold"/>
      <Action name="format_text_italic"/>
      <Action name="format_text_underline"/>
      <Action name="format_text_strikeout"/>
      <Action name="format_text_foreground_color"/>
      <Action name="format_text_background_color"/>
      <Separator lineSeparator="true"/>
      <Action name="format_align_left"/>
      <Action name="format_align_center"/>
      <Action name="format_align_right"/>
      <Action name="format_align_justify"/>
      <Separator lineSeparator="true"/>
      <Action name="format_list_style"/>
      <Action name="format_list_indent_more"/>
      <Action name="format_list_indent_less"/>
      <Separator lineSeparator="true"/>
      <Action name="insert_horizontal_rule"/>
      <Action name="manage_link"/>
      <Action name="format_painter"/>
  </Menu>

  <Menu name="go">
   <text>&amp;Go</text>
   <Action name="go_next_page"/>
   <Action name="go_prev_page"/>
   <Separator/>
   <Action name="go_prev_book"/>
   <Action name="go_next_book"/>
  </Menu>

  <Menu name="tools">
   <text>&amp;Tools</text>
   <Action name="insert_date"/>
   <Action name="copyIntoTitle"/>
   <Action name="linkify"/>
  </Menu>
  
  <Action name="bookmarks"/>

  <Menu noMerge="1" name="settings">
    <text>&amp;Settings</text>
	<Action name="kjots_configure" group="settings_configure" />
  </Menu>

  <Menu name="help"></Menu>
 </MenuBar>

 <ToolBar name="mainToolBar" fullWidth="true" noMerge="1" iconText="icononly">
  <text>Main Toolbar</text>
  <Separator/>
  <Action name="file_print"/>
  <Separator/>
  <Action name="edit_cut"/>
  <Action name="edit_copy"/>
  <Action name="edit_paste"/>
  <Separator/>
  <Action name="edit_find"/>
 </ToolBar>

     
<ToolBar name="textToolBar" iconText="icononly"><text>Text Toolbar</text>
    <Action name="format_text_bold"/>
    <Action name="format_text_italic"/>
    <Action name="format_text_underline"/>
    <Action name="format_text_strikeout"/>
    <Action name="format_text_foreground_color"/>
    <Action name="format_text_background_color"/>
    <Separator lineSeparator="true"/>
    <Action name="format_font_family"/>
    <Action name="format_font_size"/>
    <Separator lineSeparator="true"/>
  <Action name="format_painter"/>
</ToolBar>
<ToolBar name="formatToolBar" iconText="icononly"><text>Format Toolbar</text>
    <Separator lineSeparator="true"/>
    <Action name="format_align_left"/>
    <Action name="format_align_center"/>
    <Action name="format_align_right"/>
    <Action name="format_align_justify"/>
    <Separator lineSeparator="true"/>
    <Action name="format_list_style"/>
    <Action name="format_list_indent_more"/>
    <Action name="format_list_indent_less"/>
    <Separator lineSeparator="true"/>
    <Action name="insert_horizontal_rule"/>
    <Action name="manage_link"/>
</ToolBar>
</kpartgui>
